% Procedure MarkovMaleFemale.m
% Inhomogenous markov chain for sexual reproduction with variation outside
% the sex chromosome.
% Ref.: P.A. Tyvand & S. Thorvaldsen (2007): A sexually neutral discrete Markov model 
% for given sum males + females. Theoretical Population Biology, 73, 148-152. 
% Chromosomes: A and a in the male population, and Ao ao in the female.
% NB: A=Ao and a=ao, so the index o is only used as a marker for sex
% We then have six genotypes: AA, Aa, aa, AoAo, Aoao, aoao with size n(1..6).

% Algorithm:
% Read input parameters (founder population V0 (with population size npop) and size of the comming populations, Npop)
% For generation t = 1: T
%   Determine row j in transition matrix M 
%   V(t+1)= V(t)*Mj
%   compute statistics in the generation
% end
% Write statistics

clear all;

g=6; % The number of different genotypes in this population

%%%%%%%%%%%% INPUT DATA that may be changed: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the initial founder population of each genotype:
n(1)=1;
n(2)=1;
n(3)=0;
n(4)=2;
n(5)=0;
n(6)=0;
% Giving the population size of the comming generations (this input may be changed):
%Npop=[8, 8, 4, 4, 4, 8, 8, 8, 4, 4, 4, 8, 8, 8, 4, 4, 4, 8, 8, 8, 4, 4, 4, 8, 8, 8, 4, 4, 4, 8, 8, 8, 4, 4, 4, 8, 8, 8, 4, 4, 4, 8, 8, 8, 4, 4, 4, 8, 8, 8, 4, 4, 4]; % for bottlenecs
%Npop=[12, 12, 3, 3, 3, 12, 12, 12, 3, 3, 3, 12, 12, 12, 3, 3, 3, 12, 12, 12, 3, 3, 3]; % for bottlenecs
Npop=4*ones(1,50);  % can alternatively be used for constant population size
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
disp ('Model with sexually neutral random mating.');
disp ('Founder population of 3 + 3 genotypes.  Male: AA Aa aa. Female: AoAo Aoao aoao. '); n
npop=sum(n);
d=(factorial(g+npop-1))/(factorial(npop)*factorial(g-1)); % generally valid for g genotypes
Vinput=zeros(1,d);
% To compute the index i of the founder:
i=icompute(n)
Vinput(i)=1;

disp (['Population size in the subsequent generations:  ', num2str(Npop)])
T=length(Npop); % The number of comming generations

% variables for storing data from each generation, in tabular form (generations vertical):
% index can't be 0 in Matlab, so founder generation has to be No 1
% v=zeros(T+1,d); % can be used to store v, if nessesary.
% v(1,:)=vinput;
Efreq=zeros(T+1,g);
SD=zeros(T+1,g);
Efix=zeros(T+1,2);
Efreq(1,:)=[n(1)/npop, n(2)/npop, n(3)/npop, n(4)/npop, n(5)/npop, n(6)/npop];

% looping through the T generations:
for t=1:T
    disp (['Starting generation: ', num2str(t),'... '])
    % must determine the distribution in next population, pluss statistics:
    [Vnew, Efreq(t+1,:), SD(t+1,:), Efix(t+1,:)] = Vcomp6g(Vinput, npop, Npop(t));
    % The next population distribution is given by:
    Vinput=Vnew;
    npop=Npop(t);
    % V(t+1,:)=Vnew; % NB: must match in dimention
end
disp ('Frequencies of the different genotypes:'); Efreq
disp ('Standard Deviation:'); SD
disp ('Probability of fixation in A and a:'); Efix
% V
    


