function [] = Vplot (V, mask);
% A function to plot the distribution vector over its generations

% V: Cell array containing population distribution vectors covering all 
% different compositions of gamete populations.
% mask: the resolution of plot surface 
% Use: [] = Vplot (v, mask);

% The function is made flexible conserning input parameters:
if nargin < 2
    mask=1;% default resolution for plotting, in inteval: <0.1 to 2]
end
% To make it possible to use array (not only cell arrays) as input: 
%if isnumeric(V)...

% Reconstructing the length of the data:
T = size(V,2); % the number of vectors (or generations)
IV=zeros(T,1);%'int32'); %max index
n=zeros(T,1); %population size
for t=1:T
    IV(t)=size(V{t},2) % the number of indices in each vector FUNKER IKKE!!
end
n=(IV-1)/2; %population size in generation t
MaxIV=max(IV);
TotIV=sum(IV);
% The vectors x, y, z to contain nonuniformly data: 
x=zeros(TotIV,1); y=zeros(TotIV,1); z=zeros(TotIV,1);
j=0;
for t=1:T
    xt=zeros(IV(t),1);
    for i=1:IV(t)
        xt(i)=(double(i)-1)/(2*n(t)); %converts to real numbers
    end
    x(j+1:j+IV(t))=xt; %dimensionless coordinate
    y(j+1:j+IV(t))=double(t); %the time variable
    z(j+1:j+IV(t))=V{t}; %the  diffusion variable in [0,1]
    j=j+IV(t);
end

% x, y, and z are vectors containing non-uniformly sampled data. 
% We define a regular grid, and grid the data to it: 
ti = 1:T;
ni = 0:1/(MaxIV/mask):1;
[XI,YI] = meshgrid(ni,ti);
ZI = griddata(x,y,z,XI,YI,'linear'); %use Delaunay triangle, 
%optional methods: 'linear' 'nearest 'cubic' or 'v4' 
%   cubic may give som irritating negative values...
%ZI = interp2(x,y,z,XI,YI); %not possible to use Veronoi polygon here

% Plot the gridded data along with the nonuniform data points used to generate it: 
mesh(XI,YI,ZI); hold on
plot3(x,y,z,'.');
hold off
ylabel ('Generations');
axis([0 1 0 T 0 1])