% Procedure MaAa.m
% Inhomogenous markov chain for asexual random reproduction with two different
% alleles, A and a, in one locus.
% We here have three genotypes: AA, Aa, aa with size n(1), n(2), n(3).
% Ref.: Tyvand, P.A.: An exact algebraic theory of genetic drift in finite
% diploid populations with random mating. J. theor. Biol. 163, 315-331. 1993.


% Algorithm:
% Read input parameters (founder population V0 (with population size npop) and size of the comming populations, Npop)
% For generation t = 1: T
%   Determine row j in transition matrix M 
%   V(t+1)= V(t)*Mj
%   compute statistics in the generation
% end
% Write statistics

clear all;
g=3; % The number of different genotypes in this population (the input may be changed)

% Generating the initial founder population of each genotype, values may be changed:
n(1)=0;
n(2)=16;
n(3)=0;
disp ('Founder population vector of genotypes  AA Aa aa:'); n
npop=sum(n);
d=(npop+2)*(npop+1)/2; % valid for g=3 genotypes
Vinput=zeros(1,d);
i=1+n(1)+(n(1)+n(2))*(n(1)+n(2)+1)*0.5;
Vinput(i)=1;

% Giving the population size of the comming generations (this input may be changed):
%Npop=[8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2];
Npop=16*ones(1,60); % can alternatively be used for constant population size
disp ('Population size in the subsequent generations:'); Npop
T=length(Npop); % The number of comming generations

% variables for storing data from each generation, in tabular form (generations vertical):
% index can't be 0 in Matlab, so founder generation has to be No 1
% v=zeros(T+1,d); % can be used to store v, if nessesary.
% v(1,:)=vinput;
Efreq=zeros(T+1,g);
SD=zeros(T+1,g);
Efix=zeros(T+1,2);
Efreq(1,:)=[n(1)/npop, n(2)/npop, n(3)/npop];

% looping through the T generations:
for t=1:T
    s=0; % no self-mating
    % must determine the distribution in next population, pluss statistics:
    [Vnew, Efreq(t+1,:), SD(t+1,:), Efix(t+1,:)] = Vcomp2dip(Vinput, npop, Npop(t), s);
    % The next population distribution is given by:
    Vinput=Vnew;
    npop=Npop(t);
end
disp ('Frequencies of the different genotypes:'); Efreq
disp ('Standard Deviation:'); SD
disp ('Probability of fixation in XY and Xy:'); Efix

    


