% Procedure WrightFisherHaploid.m
% Non-homogenous markov chain for asexual random reproduction in a diploid 
% population with two different alleles, A and a, in one locus.
% We here considere the GAMETE pool: A and a with size n(1), n(2).
% Mutation rates epsi, and selection rates beta are given.
% Ref.: Tyvand, P.A. and Thorvaldsen, S. (2010): Exact Markov chains versus 
% diffusion theory for haploid random mating. Mathematical Bioscienses, 225, 18-23.

% Algorithm:
% Read input parameters (gametes in founder population V0),and size (Npop), 
% mutation rate and selection rates in the coming populations
% For generation t = 1: T
%   Determine row j in transition matrix M 
%   V(t+1)= V(t)*Mj
%   compute statistics in the generation
% end
% Write statistics

clear all;
g=2; %the number of different gametes

%%%%%%%%%%%% INPUT DATA that may be changed: %%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the initial founder population of each gamete, 
% (remember that the sum must be an even number):
n(1)=48;
n(2)=32;
%T=50; % The number of coming generations
% Giving the number of diploid individuals in the subsequent generations:
%Npop=10*ones(1,T);  % for constant population size
Npop=[80 80 10 10 80 80 10 10 80 80 10 10 80 80 10 10 80 80 10 10];
%Npop=[40 5 40 5 40 5 40 5 40 5 40 5 40 5 40 5 40 5 40 5]; 
%Npop=[40 40 40 40 20 20 20 20 10 10 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5]; % can alternatively be used
T=length(Npop); % The assigned number of comming generations
% Giving the mutation rate from A to a, and a to A, in the subsequent generations:
%delta=[0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01];
%epsi=[0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01];
delta=0*ones(1,T);  % can alternatively be used for constant mutation rate A to a
epsi=0*ones(1,T);  % can alternatively be used for constant mutation rate a to A
% Giving the selection rate for alleles a and A in the subsequent generations:
%w=[2 2 2 2 2 2 2 2 2 2 2 2 2 2 2;
%   2 2 2 2 2 2 2 2 2 2 2 2 2 2 2];
% can alternatively be used for constant selection rate
w(1,:)=1*ones(1,T);  % constant fitness rate for A
w(2,:)=1*ones(1,T);  % constant fitness rate for a
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
format short;
disp ('Discrete Markov model of haploid micro-evoluiton');
disp (['Founder population vector of gametes A and a: ', num2str(n)])
npop=sum(n)/2;
% The number of different diploid population compositions:
d=2*npop+1; % the number of different gamete populations
Vinput=zeros(1,d);
i=n(1)+1;
Vinput(i)=1;
disp (['Population size in the subsequent generations:  ', num2str(Npop)])
disp (['Fitness rate for A in the subsequent generations:  ', num2str(w(1,:))])
disp (['Fitness rate for a in the subsequent generations:  ', num2str(w(2,:))]);
disp (['Mutation rate from allele A to a in the subsequent generations:  ', num2str(delta)]); 
disp (['Mutation rate from allele a to A in the subsequent generations:  ', num2str(epsi)]); 

% variables for storing data from each generation, in tabular form (generations vertical):
% index can't be 0 in Matlab, so founder generation has to be No 1
% v=zeros(T+1,d); % can be used to store v, if nessesary.
% v(1,:)=vinput;
Efreq=zeros(T+1,g);
SD=zeros(T+1,g);
Efix=zeros(T+1,2);
Efreq(1,:)=[n(1)/(2*npop), n(2)/(2*npop)];
%v{1}=Vinput; %use cell array to store data of unequal size
% looping through the T generations:
for t=1:T
    disp (['Starting generation: ', num2str(t),'... '])
    % must determine the distribution in next population, pluss statistics:
    [Vnew, Efreq(t+1,:), SD(t+1,:), Efix(t+1,:)] = VcompWrightFisherHaploid(Vinput, npop, Npop(t),delta(t),epsi(t),w(:,t));
    format long;
    v{t}=Vnew; %use cell array to store data of unequal size
    Vnew
    %vmat=cell2num(v{t}) %converting to regular matrix
    % The next population distribution is given by:
    Vinput=Vnew;
    npop=Npop(t);
end
%vmat=cell2mat(v(:)); %converting to regular matrix
Vplot (v, 0.5); %plotting distribution vector over its generations, mask resolution 0.5
%format long;
%v
%disp ('Frequencies of the different gametes:'); Efreq
%disp ('Standard Deviation:'); SD
%disp ('Probability of fixation in A and in a:'); Efix

    


