% Procedure MaAa.m
% Inhomogenous markov chain for sexual reproduction with variation outside
% the sex chromosome. Asume given number of males and females.
% Reference: PA Tyvand and S. Thorvaldsen (2010): A BISEXUAL MARKOV MODEL 
% WITH THE NUMBERS OF MALES AND FEMALES GIVEN. Manuscript.
% Chromosomes: A and a in each sex.
% We then have three genotypes AA, Aa, aa in each sex: male m(1..3) and female n(1..3)

% Algorithm:
% Read input parameters (founder population V0 (with population size mpop and npop) 
% and size of the comming populations, Mpop and Npop)
% For generation t = 1: T
%   Determine transition matrix M 
%   V(t+1)= V(t)*M
%   compute statistics in the generation
% end
% Write statistics

clear all;

g=3; % The number of different genotypes in this population

%%%%%%%%%%%% INPUT DATA that may be changed: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the initial male and female founder population of each genotype:
m(1)=0;
m(2)=4;
m(3)=0;
n(1)=0;
n(2)=4;
n(3)=0;
% Giving the population size of males and females in the comming generations 
% (this input may be changed but must be of same length):
Mpop=[2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4,2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4]; % for bottlenecs
Npop=[2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4,2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4]; % for bottlenecs
%Mpop=[4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8];
%Npop=[4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8];
%Mpop=8*ones(1,100);  % can alternatively be used for constant population size of males
%Npop=8*ones(1,100);
T=length(Mpop); % The number of comming generations
d1=0*ones(1,T); % the mutation rate from allele A to a 
e1=0*ones(1,T); % the mutation rate from allele a to A 
d2=0*ones(1,T); % the mutation rate from allele Ao to ao (females)
e2=0*ones(1,T); % e2 the mutation rate from allele ao to Ao (females)
w1=1*ones(1,T); % fitness parameter for genotype AA (in males)
w2=1*ones(1,T); % for genotype Aa
w3=1*ones(1,T); % for genotype aa
w4=1*ones(1,T); % for genotype AoAo (in females)
w5=1*ones(1,T); % for genotype Aoao
w6=1*ones(1,T); % for genotype aoao
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
disp ('Model with sexually neutral random mating with mutatin and fitness. Number of males and females given');
disp (['Founder population of 3 & 3 genotypes.  Male: AA Aa aa: ', num2str(m),' Female: AA Aa aa: ', num2str(n)])
mpop=sum(m);
npop=sum(n);
dm=(mpop+1)*(mpop+2)/2;
dn=(npop+1)*(npop+2)/2;
Vinput=zeros(dm,dn);
% To compute the index i and j of the founder:
i=icompute(m)
j=icompute(n)
Vinput(i,j)=1;

disp ('Population size in the subsequent generations: ')
disp (['Males:    ', num2str(Mpop)])
disp (['Females:  ', num2str(Npop)])


% variables for storing data from each generation, in tabular form (generations vertical):
% index can't be 0 in Matlab, so founder generation has to be No 1
% v=zeros(T+1,d); % can be used to store v, if nessesary.
% v(1,:)=vinput;
Efreq=zeros(T+1,6);
SD=zeros(T+1,6);
Efix=zeros(T+1,2);
Efreq(1,:)=[m(1)/mpop, m(2)/mpop, m(3)/mpop, n(1)/npop, n(2)/npop, n(3)/npop];

% looping through the T generations:
for t=1:T
    disp (['Starting generation: ', num2str(t),'... '])
    % To determine the distribution in next population, pluss statistics:
    [Vnew, Efreq(t+1,:), SD(t+1,:), Efix(t+1,:)] = VcompMF6g(Vinput, mpop, npop, Mpop(t), Npop(t), d1(t), e1(t), d2(t), e2(t), w1(t), w2(t), w3(t), w4(t), w5(t), w6(t));
    % The next population distribution is given by:
    Vinput=Vnew;
    mpop=Mpop(t);
    npop=Npop(t);
end
format long;
disp ('Frequencies of the different genotypes:'); single(Efreq)
disp ('Standard Deviation:'); single(SD)
disp ('Probability of fixation in A and a:'); single(Efix)
