% Procedure WrightFisherSocialInsects.m
% Inhomogenous markov chain for sexual reproduction with haploid males and
% diploid females (called social insects).
% Asume given number of males and females.
% Chromosomes: A and a in each sex.
% We then have two male genotypes A, a; and three female AA, Aa, aa

% Reference:
% PA Tyvand and S Thorvaldsen (2010): WrightFisher model of social insects with
% haploid males and diploid females. Journal of Theoretical Biology 266,470478

% Algorithm:
% Read input parameters (founder population V0 (with population size mpop and npop) 
% and size of the comming populations, Mpop and Npop)
% For generation t = 1: T
%   Determine transition matrix M 
%   V(t+1)= V(t)*M
%   compute statistics in the generation
% end
% Write statistics

clear all;
g=5; % The number of different genotypes in this population

%%%%%%%%%%%% INPUT DATA that may be changed: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the initial male and female founder population of each genotype:
m(1)=6;
m(2)=2;
n(1)=2;
n(2)=0;
n(3)=2;
% Giving the population size of males and females in the comming generations 
% (this input may be changed but Mpop and Npop must be of same length):
%Mpop=[2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4,2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4]; % for bottlenecs
%Npop=[2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4,2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4, 2, 4]; % for bottlenecs
%Npop=[8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2];
Npop=[4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4];%, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8];
Mpop=[8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2, 8, 8, 8, 2, 2, 2];%, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8, 4, 4, 8, 8];
%Mpop=4*ones(1,25);  % can alternatively be used for constant population size of males
%Npop=4*ones(1,25);
T=length(Mpop); % The number of comming generations
d1=0*ones(1,T); % the mutation rate from male gamete A to a 
e1=0*ones(1,T); % the mutation rate from male gamete a to A 
d2=0*ones(1,T); % the mutation rate from female gamete Ao to ao
e2=0*ones(1,T); % the mutation rate from female gamete ao to Ao
w1=1*ones(1,T); % fitness parameter for survival selection for genotype A (in males). 
w2=1*ones(1,T); % for genotype a
w3=1*ones(1,T); % for genotype AA (in females)
w4=1*ones(1,T); % for genotype Aa
w5=1*ones(1,T); % for genotype aa
dd1=0*ones(1,T); % the late mutation rate from male gamete A to a 
ee1=0*ones(1,T); % the late mutation rate from male gamete a to A 
dd2=0*ones(1,T); % the late mutation rate from female gamete Ao to ao
ee2=0*ones(1,T); % the late mutation rate from female gamete ao to Ao
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
disp ('WrightFisher model of social insects. Number of males and females given');
disp (['Founder population of 2 & 3 genotypes.  Male: A a: ', num2str(m),' Female: AA Aa aa: ', num2str(n)])
mpop=sum(m);
npop=sum(n);
dm=(mpop+1);
dn=(npop+1)*(npop+2)/2;
Vinput=zeros(dm,dn);
% To compute the index i and j of the founder:
i=icompute(m)
j=icompute(n)
Vinput(i,j)=1;

disp ('Population size in the subsequent generations: ')
disp (['Males:    ', num2str(Mpop)])
disp (['Females:  ', num2str(Npop)])


% variables for storing data from each generation, in tabular form (generations vertical):
% index can't be 0 in Matlab, so founder generation has to be No 1
% v=zeros(T+1,d); % can be used to store v, if nessesary.
% v(1,:)=vinput;
Efreq=zeros(T+1,5);
SD=zeros(T+1,5);
Efix=zeros(T+1,2);
Efreq(1,:)=[m(1)/mpop, m(2)/mpop, n(1)/npop, n(2)/npop, n(3)/npop];

% looping through the T generations:
for t=1:T
    disp (['Starting generation: ', num2str(t),'... '])
    % To determine the distribution in next population, pluss statistics:
    [Vnew, Efreq(t+1,:), SD(t+1,:), Efix(t+1,:)] = VcompWF5g(Vinput, mpop, npop, Mpop(t), Npop(t), d1(t), e1(t), d2(t), e2(t), w1(t), w2(t), w3(t), w4(t), w5(t), dd1(t), ee1(t), dd2(t), ee2(t));
    % The next population distribution is given by:
    Vinput=Vnew;
    mpop=Mpop(t);
    npop=Npop(t);
end
format long;
disp ('Frequencies of the different genotypes:'); single(Efreq)
disp ('Standard Deviation:'); single(SD)
disp ('Probability of fixation in A and a:'); single(Efix)
